"""Slack attachment builders."""
from datetime import timedelta
import json
import re
import socket
import time

from billiard.process import current_process
from celery import __version__ as CELERY_VERSION
from celery.schedules import crontab

if CELERY_VERSION >= "4.0.0":
    from celery.schedules import solar


STOPWATCH = {}
BEAT_DELIMITER = " -> "


def add_task_to_stopwatch(task_id):
    """Add a task_id to the STOPWATCH dict."""
    STOPWATCH[task_id] = time.time()


def get_task_prerun_attachment(task_id, task, args, kwargs, **cbkwargs):
    """Create the slack message attachment for a task prerun."""

    if (cbkwargs["exclude_tasks"] and
            any([re.search(task, task_name)
                for task in cbkwargs["exclude_tasks"]])):
        STOPWATCH.pop(task_id)
        return
    elif (cbkwargs["include_tasks"] and
            not any([re.search(task, task_name)
                    for task in cbkwargs["include_tasks"]])):
        STOPWATCH.pop(task_id)
        return
    
    message = "Executing -- " + task.name.rsplit(".", 1)[-1]

    lines = ["Name: *" + task.name + "*"]

    if cbkwargs["show_task_id"]:
        lines.append("Task ID: " + task_id)

    if cbkwargs["use_fixed_width"]:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + "`" + str(args) + "`")
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + "`" + str(kwargs) + "`")
    else:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + str(args))
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + str(kwargs))

    executing = "\n".join(lines)

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": cbkwargs["slack_task_prerun_color"],
                "text": executing,
                "title": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    if cbkwargs["flower_base_url"]:
        attachment["attachments"][0]["title_link"] = (
            cbkwargs["flower_base_url"] +
            "/task/{tid}".format(tid=task_id)
        )

    return attachment


def get_task_success_attachment(task_name, retval, task_id,
                                args, kwargs, **cbkwargs):
    """Create the slack message attachment for a task success."""
    if (cbkwargs["exclude_tasks"] and
            any([re.search(task, task_name)
                for task in cbkwargs["exclude_tasks"]])):
        STOPWATCH.pop(task_id)
        return
    elif (cbkwargs["include_tasks"] and
            not any([re.search(task, task_name)
                    for task in cbkwargs["include_tasks"]])):
        STOPWATCH.pop(task_id)
        return

    if isinstance(retval, dict):
        retval = json.dumps(retval, indent=4)
    else:
        retval = str(retval)

    message = "SUCCESS -- " + task_name.rsplit(".", 1)[-1]

    elapsed = divmod(time.time() - STOPWATCH.pop(task_id), 60)

    lines = ["Name: *" + task_name + "*"]

    if cbkwargs["show_task_execution_time"]:
        lines.append("Execution time: {m} minutes {s} seconds".format(
            m=str(int(elapsed[0])),
            s=str(int(elapsed[1])),
        ))
    if cbkwargs["show_task_id"]:
        lines.append("Task ID: " + task_id)

    if cbkwargs["use_fixed_width"]:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + "`" + str(args) + "`")
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + "`" + str(kwargs) + "`")
        if cbkwargs["show_task_return_value"]:
            lines.append("Returned: " + "```" + str(retval) + "```")
    else:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + str(args))
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + str(kwargs))
        if cbkwargs["show_task_return_value"]:
            lines.append("Returned: " + str(retval))

    success = "\n".join(lines)

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": cbkwargs["slack_task_success_color"],
                "text": success,
                "title": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    if cbkwargs["flower_base_url"]:
        attachment["attachments"][0]["title_link"] = (
            cbkwargs["flower_base_url"] +
            "/task/{tid}".format(tid=task_id)
        )

    return attachment

def get_task_duplicate_attachment(task_name, retval, task_id,
                                args, kwargs, **cbkwargs):
    """Create the slack message attachment for a task duplicate state."""
    if (cbkwargs["exclude_tasks"] and
            any([re.search(task, task_name)
                for task in cbkwargs["exclude_tasks"]])):
        STOPWATCH.pop(task_id)
        return
    elif (cbkwargs["include_tasks"] and
            not any([re.search(task, task_name)
                    for task in cbkwargs["include_tasks"]])):
        STOPWATCH.pop(task_id)
        return

    if isinstance(retval, dict):
        retval = json.dumps(retval, indent=4)
    else:
        retval = str(retval)

    message = "DUPLICATE -- " + task_name.rsplit(".", 1)[-1]

    elapsed = divmod(time.time() - STOPWATCH.pop(task_id), 60)

    lines = ["Name: *" + task_name + "*"]

    if cbkwargs["show_task_execution_time"]:
        lines.append("Execution time: {m} minutes {s} seconds".format(
            m=str(int(elapsed[0])),
            s=str(int(elapsed[1])),
        ))
    if cbkwargs["show_task_id"]:
        lines.append("Task ID: " + task_id)

    if cbkwargs["use_fixed_width"]:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + "`" + str(args) + "`")
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + "`" + str(kwargs) + "`")
        if cbkwargs["show_task_return_value"]:
            lines.append("Returned: " + "```" + str(retval) + "```")
    else:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + str(args))
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + str(kwargs))
        if cbkwargs["show_task_return_value"]:
            lines.append("Returned: " + str(retval))

    success = "\n".join(lines)

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": cbkwargs["slack_task_duplicate_color"],
                "text": success,
                "title": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    if cbkwargs["flower_base_url"]:
        attachment["attachments"][0]["title_link"] = (
            cbkwargs["flower_base_url"] +
            "/task/{tid}".format(tid=task_id)
        )

    return attachment


def get_task_failure_attachment(task_name, exc, task_id, args,
                                kwargs, einfo, **cbkwargs):
    """Create the slack message attachment for task failure."""
    if (cbkwargs["exclude_tasks"] and
            any([re.search(task, task_name)
                for task in cbkwargs["exclude_tasks"]])):
        STOPWATCH.pop(task_id)
        return
    elif (cbkwargs["include_tasks"] and
            not any([re.search(task, task_name)
                    for task in cbkwargs["include_tasks"]])):
        STOPWATCH.pop(task_id)
        return

    message = "FAILURE -- " + task_name.rsplit(".", 1)[-1]

    elapsed = divmod(time.time() - STOPWATCH.pop(task_id), 60)

    lines = ["Name: *" + task_name + "*"]

    if cbkwargs["show_task_execution_time"]:
        lines.append("Execution time: {m} minutes {s} seconds".format(
            m=str(int(elapsed[0])),
            s=str(int(elapsed[1])),
        ))
    if cbkwargs["show_task_id"]:
        lines.append("Task ID: " + task_id)

    if cbkwargs["use_fixed_width"]:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + "`" + str(args) + "`")
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + "`" + str(kwargs) + "`")
        lines.append("Exception: " + "`" + str(exc) + "`")
        if cbkwargs["show_task_exception_info"]:
            lines.append("Info: " + "```" + str(einfo) + "```")
    else:
        if cbkwargs["show_task_args"]:
            lines.append("args: " + str(args))
        if cbkwargs["show_task_kwargs"]:
            lines.append("kwargs: " + str(kwargs))
        lines.append("Exception: " + str(exc))
        if cbkwargs["show_task_exception_info"]:
            lines.append("Info: " + str(einfo))

    failure = "\n".join(lines)

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": cbkwargs["slack_task_failure_color"],
                "text": failure,
                "title": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    if cbkwargs["flower_base_url"]:
        attachment["attachments"][0]["title_link"] = (
            cbkwargs["flower_base_url"] +
            "/task/{tid}".format(tid=task_id)
        )

    return attachment


def get_celery_startup_attachment(**kwargs):
    """Create the slack message attachment for celery startup."""
    if kwargs["show_celery_hostname"]:
        message = "*Celery is starting up on {}.*".format(
            socket.gethostname()
        )
    else:
        message = "*Celery is starting up.*"

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": kwargs["slack_celery_startup_color"],
                "text": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    return attachment


def get_celery_shutdown_attachment(**kwargs):
    """Create the slack message attachment for celery shutdown."""
    if kwargs["show_celery_hostname"]:
        message = "_Celery is shutting down on {}._".format(
            socket.gethostname()
        )
    else:
        message = "_Celery is shutting down._"

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": kwargs["slack_celery_shutdown_color"],
                "text": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    return attachment


def get_beat_init_attachment(**kwargs):
    """Create the slack message attachment for beat init."""
    if kwargs["show_celery_hostname"]:
        message = "*Beat is (re)initializing on {}*".format(
            socket.gethostname()
        )
    else:
        message = "*Beat is (re)initializing*"

    beat_schedule = kwargs["beat_schedule"]
    if beat_schedule:
        message += " *with schedule:*"

        sched = []
        for task in sorted(beat_schedule):
            if kwargs["beat_show_full_task_path"]:
                sched.append(
                    task + BEAT_DELIMITER +
                    schedule_to_string(beat_schedule[task]["schedule"]))
            else:
                sched.append(
                    task.split(".", 2)[-1] + BEAT_DELIMITER +
                    schedule_to_string(beat_schedule[task]["schedule"]))
        schedule = (
            "\n*Task{}crontab(m/h/d/dM/MY) OR solar OR interval:*\n".format(
                BEAT_DELIMITER
            ) +
            "```" + "\n".join(sched) + "```"
        )
    else:
        message = message[:-1] + "." + message[-1:]
        schedule = ""

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": kwargs["slack_beat_init_color"],
                "text": message + schedule,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    return attachment


processes = {
    "MainProcess": "Celery",
    "Beat": "Beat"
}


def get_broker_disconnect_attachment(**kwargs):
    """Create the slack message attachment for broker disconnection."""
    if kwargs["show_celery_hostname"]:
        message = "*{process} could not connect to broker on {host}.*".format(
            process=processes.get(
                current_process()._name, current_process()._name),
            host=socket.gethostname()
        )
    else:
        message = "*{process} could not connect to broker.*".format(
            process=processes.get(
                current_process()._name, current_process()._name),
        )

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": kwargs["slack_broker_disconnect_color"],
                "text": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    return attachment


def get_broker_connect_attachment(**kwargs):
    """Create the slack message attachment for broker connection."""
    if kwargs["show_celery_hostname"]:
        message = "*{process} (re)connected to broker on {host}.*".format(
            process=processes.get(
                current_process()._name, current_process()._name),
            host=socket.gethostname()
        )
    else:
        message = "*{process} (re)connected to broker.*".format(
            process=processes.get(
                current_process()._name, current_process()._name),
        )

    attachment = {
        "attachments": [
            {
                "fallback": message,
                "color": kwargs["slack_broker_connect_color"],
                "text": message,
                "mrkdwn_in": ["text"]
            }
        ],
        "text": ""
    }

    return attachment


def schedule_to_string(schedule):
    """Transform a crontab, solar, or timedelta to a string."""
    if isinstance(schedule, crontab):
        return str(schedule)[10:-15]
    elif CELERY_VERSION >= "4.0.0" and isinstance(schedule, solar):
        return str(schedule)[8:-1]
    elif isinstance(schedule, timedelta):
        return "every " + str(schedule)
    else:
        return "every " + str(schedule) + " seconds"



"""Post to slack."""
import logging
import time

import requests
from requests.exceptions import RequestException


RETRY_AFTER = 0
RATE_LIMITED = False
TIMEOUT = 1


def post_to_slack(webhook, text=" ", attachment=None, payload={}):
    """Post a message to the slack channel."""
    global RETRY_AFTER
    global RATE_LIMITED

    if time.time() <= RETRY_AFTER:
        logging.error("Slack: HTTP 429 Too Many Requests.")
        return False
    else:
        if RATE_LIMITED:
            RATE_LIMITED = False
            return post_warning_to_slack(webhook, text, attachment)

    payload.update({"text": text if text != "" else " "})

    if attachment is not None:
        payload.update(attachment)
    try:
        response = None
        response = requests.post(webhook, json=payload, timeout=TIMEOUT)
        response.raise_for_status()
    except RequestException as exc:
        logging.error("Unable to post to Slack; {e}: {msg}".format(
            e=type(exc).__name__, msg=str(exc)))
        if response:
            RETRY_AFTER = \
                time.time() + int(response.headers.get("Retry-After", 0))
            RATE_LIMITED = True

    return response


def post_warning_to_slack(webhook, text, attachment=None):
    """Post a warning to Slack about being rate limited."""
    message = (
        "*WARNING: HTTP 429 Too Many Requests from celery-slack.*\n\n"
        "*You should make adjustments to your schedule and/or restrict "
        "the set of tasks associated with celery-slack before the application "
        "becomes permanently rate-limited or disabled.*"
    )
    warning = {
        "fallback": message,
        "color": "#000000",
        "text": message,
        "mrkdwn_in": ["text"]
    }
    if attachment is None:
        attachment = {
            "attachments": [
                warning
            ],
            "text": ""
        }
    else:
        attachment["attachments"].append(warning)

    payload = {"text": ""}
    payload.update(attachment)
    try:
        response = None
        response = requests.post(webhook, json=payload, timeout=TIMEOUT)
        response.raise_for_status()
    except RequestException as exc:
        logging.error("Unable to post to Slack; {e}: {msg}".format(
            e=type(exc).__name__, msg=str(exc)))

    return response



"""Module level accessible objects."""
from .__version__ import __version__
from .__version__ import __description__
from .__version__ import __url__
from .__version__ import __title__
from .__version__ import __author__
from .__version__ import __author_email__
from .__version__ import __license__
from .__version__ import __copyright__
from .__version__ import __docs_copyright__
from .slackify import DEFAULT_OPTIONS
from .slackify import Slackify

__all__ = (
    Slackify,
    DEFAULT_OPTIONS,
    __version__,
    __description__,
    __url__,
    __title__,
    __author__,
    __author_email__,
    __license__,
    __copyright__,
    __docs_copyright__,
)



"""Slackify celery."""
import re

from celery.signals import beat_init
from celery.signals import celeryd_init
from celery.signals import task_prerun
from celery.signals import worker_shutdown
import kombu

from .callbacks import slack_beat_init
from .callbacks import slack_broker_connect
from .callbacks import slack_broker_disconnect
from .callbacks import slack_celery_shutdown
from .callbacks import slack_celery_startup
from .callbacks import slack_task_failure
from .callbacks import slack_task_prerun
from .callbacks import slack_task_success
from .exceptions import InvalidColorException
from .exceptions import MissingWebhookException
from .exceptions import TaskFiltrationException
from celery_slack import slack


DEFAULT_OPTIONS = {
    "slack_beat_init_color": "#FFCC2B",
    "slack_broker_connect_color": "#36A64F",
    "slack_broker_disconnect_color": "#D00001",
    "slack_celery_startup_color": "#FFCC2B",
    "slack_celery_shutdown_color": "#660033",
    "slack_task_prerun_color": "#D3D3D3",
    "slack_task_success_color": "#36A64F",
    "slack_task_failure_color": "#D00001",
    "slack_task_duplicate_color": "#DCDCDC",
    "slack_request_timeout": 1,
    "flower_base_url": None,
    "show_celery_hostname": False,
    "show_task_id": True,
    "show_task_execution_time": True,
    "show_task_args": True,
    "show_task_kwargs": True,
    "show_task_exception_info": True,
    "show_task_return_value": True,
    "show_task_prerun": False,
    "show_startup": True,
    "show_shutdown": True,
    "show_beat": True,
    "show_broker": False,
    "use_fixed_width": True,
    "include_tasks": None,
    "exclude_tasks": None,
    "failures_only": False,
    "webhook": None,
    "beat_schedule": None,
    "beat_show_full_task_path": False,
    "channel": "#alerts-celery", 
    "username": "celery",
    "default_emoji": ":celery:",
    "success_emoji": ":check_green:",
    "failure_emoji": ":red_circle:"
}

COLOR_REGEX = r"^#[a-fA-F0-9]{6}$"


class Slackify(object):
    """Add slack callbacks to celery signals.

    :param app: An instantiation of a celery.Celery object.
    :param slack_webhook: The webhook for the slack channel in which to post.
    """

    def __init__(self, app, webhook=None, beat_schedule=None, **options):
        """Slackify the celery object."""
        self.app = app
        self.options = DEFAULT_OPTIONS.copy()
        self.options.update(**options)
        self.options["webhook"] = webhook
        self.options["beat_schedule"] = (
            beat_schedule or self.options["beat_schedule"]
        )

        if self.options["webhook"] is None:
            raise MissingWebhookException("Slack webhook must be provided.")

        if self.options["include_tasks"] and self.options["exclude_tasks"]:
            raise TaskFiltrationException(
                "Only one of 'include_tasks' and 'exclude_tasks' "
                "options can be provided.")

        colors = [self.options[c] for c in self.options.keys() if "color" in c]
        for color in colors:
            if not re.search(COLOR_REGEX, color):
                raise InvalidColorException(
                    "Color options must be hex colors.")

        self._connect_signals()
        self._decorate_task_methods()
        self._decorate_kombu_retry()

        slack.TIMEOUT = self.options["slack_request_timeout"]

    def _connect_signals(self):
        """Connect callbacks to celery signals.

        Since we are creating partials here, the weak arg must be False.
        """
        # Beat
        if self.options["show_beat"]:
            beat_init.connect(
                slack_beat_init(**self.options),
                weak=False
            )

        # Celery
        if self.options["show_startup"]:
            celeryd_init.connect(
                slack_celery_startup(**self.options),
                weak=False
            )
        if self.options["show_shutdown"]:
            worker_shutdown.connect(
                slack_celery_shutdown(**self.options),
                weak=False
            )

        # Task
        task_prerun.connect(
            slack_task_prerun(**self.options),
            weak=False
        )

    def _decorate_task_methods(self):
        """Decorate the Task class result methods."""
        if not self.options["failures_only"]:
            self.app.Task.on_success = \
                slack_task_success(**self.options)(self.app.Task.on_success)
        self.app.Task.on_failure = \
            slack_task_failure(**self.options)(self.app.Task.on_failure)

    def _decorate_kombu_retry(self):
        """Decorate the kombu.connection.retry_over_time function."""
        if self.options["show_broker"]:
            kombu.connection.retry_over_time = \
                slack_broker_disconnect(**self.options)(
                    kombu.connection.retry_over_time
                )
            kombu.connection.retry_over_time = \
                slack_broker_connect(**self.options)(
                    kombu.connection.retry_over_time
                )



"""Celery-slack exceptions."""
# flake8: noqa D103


class CelerySlackException(Exception):
    pass


class MissingWebhookException(CelerySlackException):
    pass


class TaskFiltrationException(CelerySlackException):
    pass


class InvalidColorException(CelerySlackException):
    pass



"""Version information."""
__title__ = 'celery-slack-webhooks'
__description__ = 'A Slack extension for Celery.'
__url__ = 'https://github.com/leigh-johnson/celery-slack'
__version__ = '0.4.5-rc2'
__author__ = 'Leigh Johnson'
__author_email__ = 'ljohnson@slack-corp.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017-2018 Christopher Flynn, 2019 Leigh Johnson'
__docs_copyright__ = '2017-2018 Christopher Flynn, 2019 Leigh Johnson'



"""Celery state and task callbacks."""
from functools import wraps
import time

from .attachments import get_beat_init_attachment
from .attachments import get_broker_connect_attachment
from .attachments import get_broker_disconnect_attachment
from .attachments import get_celery_shutdown_attachment
from .attachments import get_celery_startup_attachment
from .attachments import get_task_failure_attachment
from .attachments import get_task_prerun_attachment
from .attachments import get_task_success_attachment
from .attachments import add_task_to_stopwatch
from .slack import post_to_slack


def slack_task_prerun(**cbkwargs):
    """Return the task_prerun callback."""
    def slack_task_prerun_callback(task_id, task, args, kwargs, **skwargs):
        """Initialize the task timer.

        This function connects to the task_prerun signal in order to be able to
        output execution time on tasks.
        """
        add_task_to_stopwatch(task_id)

        if cbkwargs["show_task_prerun"]:

            attachment = get_task_prerun_attachment(
                task_id, task, args, kwargs, **cbkwargs)

            post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    return slack_task_prerun_callback


def slack_task_success(**cbkwargs):
    """Wrap the app.Task.on_success() method with this callback."""
    def wrapper(func):

        @wraps(func)
        def wrapped_func(self, retval, task_id, args, kwargs):
            """Post a message to slack for successful task completion.

            This function is meant to decorate app.Task.on_success where app is
            an instance of a Celery() object, thus it has the same signature.
            """

            # if not self.request.is_eager and self.AsyncResult(task_id).status == 'DUPLICATE':
            #     attachment = get_task_duplicate_attachment(
            #         self.name, retval, task_id, args, kwargs, **cbkwargs
            #     )
            # else:

            attachment = get_task_success_attachment(
                self.name, retval, task_id, args, kwargs, **cbkwargs)

            if attachment:
                post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["success_emoji"],
                "channel": cbkwargs["channel"],
            })

            return func(self, retval, task_id, args, kwargs)

        return wrapped_func

    return wrapper


def slack_task_failure(**cbkwargs):
    """Wrap the app.Task.on_failure() method with this callback."""
    def wrapper(func):

        @wraps(func)
        def wrapped_func(self, exc, task_id, args, kwargs, einfo):
            """Post a message to slack for failed task completion.

            This function is meant to patch app.Task.on_failure where app is an
            instance of a Celery() object, thus it has the same signature.
            """
            attachment = get_task_failure_attachment(
                self.name, exc, task_id, args, kwargs, einfo, **cbkwargs)

            if attachment:
                post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["failure_emoji"],
                "channel": cbkwargs["channel"],
            })

            return func(self, exc, task_id, args, kwargs, einfo)

        return wrapped_func

    return wrapper


def slack_celery_startup(**cbkwargs):
    """Return the celery_startup callback."""
    def slack_celery_startup_callback(**kwargs):
        """Post a message to slack when celery starts.

        This function is connected to the celeryd_init signal.
        """
        attachment = get_celery_startup_attachment(**cbkwargs)

        post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    return slack_celery_startup_callback


def slack_celery_shutdown(**cbkwargs):
    """Return the celery_shutdown callback."""
    def slack_celery_shutdown_callback(**kwargs):
        """Post a message to slack when celery starts.

        This function is connected to the worker_shutdown signal.
        """
        attachment = get_celery_shutdown_attachment(**cbkwargs)

        post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    return slack_celery_shutdown_callback


def slack_beat_init(**cbkwargs):
    """Return the beat_init callback."""
    def slack_beat_init_callback(**kwargs):
        """Post a message to slack when celery starts.

        This function is connected to the beat_init signal.
        """
        attachment = get_beat_init_attachment(**cbkwargs)

        post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    return slack_beat_init_callback


# Prevent spam
BROKER_COOLDOWN = 60
# Assume connected at start
BROKER_CONNECTED = True
BROKER_DISCONNECT_TIME = time.time() - BROKER_COOLDOWN
BROKER_CONNECT_TIME = time.time() - BROKER_COOLDOWN


def slack_broker_disconnect(**cbkwargs):
    """Wrap the kombu.connection.retry_over_time callback callable."""
    def slack_broker_disconnect_callback():
        """Post to slack and reset the cooldown on connect."""
        global BROKER_DISCONNECT_TIME
        global BROKER_CONNECT_TIME
        global BROKER_COOLDOWN
        global BROKER_CONNECTED

        BROKER_CONNECT_TIME = time.time() - BROKER_COOLDOWN

        if time.time() - BROKER_DISCONNECT_TIME > BROKER_COOLDOWN:
            BROKER_CONNECTED = False
            BROKER_DISCONNECT_TIME = time.time()
            attachment = get_broker_disconnect_attachment(**cbkwargs)
            post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    def wrapper(func):

        @wraps(func)
        def wrapped_func(fun, catch, args=[], kwargs={}, errback=None,
                    max_retries=None, interval_start=2, interval_step=2,
                    interval_max=30, callback=None):

            def callback_wrapper(cb_func):
                @wraps(cb_func)
                def wrapped_cb_func(*args, **kwargs):
                    slack_broker_disconnect_callback()
                    return cb_func(*args, **kwargs)
                return wrapped_cb_func

            if callback is not None:
                callback = callback_wrapper(callback)
            else:
                callback = slack_broker_disconnect_callback

            func(fun=fun, catch=catch, args=args, kwargs=kwargs,
                errback=errback, max_retries=max_retries,
                interval_start=interval_start, interval_step=interval_step,
                interval_max=interval_max, callback=callback)

        return wrapped_func

    return wrapper


def slack_broker_connect(**cbkwargs):
    """Wrap the kombu.connection.retry_over_time function."""
    def slack_broker_connect_callback():
        """Post to slack and reset the cooldown on disconnect."""
        global BROKER_DISCONNECT_TIME
        global BROKER_CONNECT_TIME
        global BROKER_COOLDOWN
        global BROKER_CONNECTED

        BROKER_DISCONNECT_TIME = time.time() - BROKER_COOLDOWN
        passed_cooldown = time.time() - BROKER_CONNECT_TIME > BROKER_COOLDOWN

        if not BROKER_CONNECTED and passed_cooldown:
            BROKER_CONNECTED = True
            BROKER_CONNECT_TIME = time.time()
            attachment = get_broker_connect_attachment(**cbkwargs)
            post_to_slack(cbkwargs["webhook"], " ", attachment, payload={
                "username": cbkwargs["username"],
                "icon_emoji": cbkwargs["default_emoji"],
                "channel": cbkwargs["channel"],
            })

    def wrapper(func):

        @wraps(func)
        def wrapped_func(fun, catch, args=[], kwargs={}, errback=None,
                    max_retries=None, interval_start=2, interval_step=2,
                    interval_max=30, callback=None):

            try:
                func(fun=fun, catch=catch, args=args, kwargs=kwargs,
                        errback=errback, max_retries=max_retries,
                        interval_start=interval_start,
                        interval_step=interval_step,
                        interval_max=interval_max, callback=callback)
            except Exception as exc:  # pragma: no cover
                raise exc

            slack_broker_connect_callback()

        return wrapped_func

    return wrapper



"""Setup script."""
import io
from os import path
from setuptools import setup


here = path.abspath(path.dirname(__file__))

# io.open for py27
with io.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

about = {}
with open(path.join(here, 'celery_slack', '__version__.py')) as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=long_description,
    url=about['__url__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    license=about['__license__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: System :: Logging',
        'Topic :: System :: Monitoring',
    ],
    keywords='celery slack',
    packages=['celery_slack'],
    install_requires=[
        'celery',
        'requests',
        'ephem',
    ],
    include_package_data=True,
)
